"""
pyap.utils
~~~~~~~~~~~~~~~~

This module provides some utility functions.

:copyright: (c) 2015 by Vladimir Goncharov.
:license: MIT, see LICENSE for more details.
"""

import re
from re import Match, RegexFlag

DEFAULT_FLAGS = re.VERBOSE | re.UNICODE


def match(regex: str, string: str, flags: RegexFlag = DEFAULT_FLAGS) -> Match | None:
    """Utility function for re.match"""
    return re.match(regex, string, flags=flags)


def findall(
    regex: str, string: str, flags: RegexFlag = DEFAULT_FLAGS
) -> list[Match | None]:
    """Utility function for re.findall"""
    return re.findall(regex, string, flags=flags)


def finditer(regex: str, string: str, flags: RegexFlag = DEFAULT_FLAGS) -> list[Match]:
    """Utility function for re.finditer"""
    return list(re.finditer(regex, string, flags=flags))



"""
pyap.address
~~~~~~~~~~~~~~~~

Contains class for constructing Address object which holds information
about address and its components.

:copyright: (c) 2015 by Vladimir Goncharov.
:license: MIT, see LICENSE for more details.
"""

from typing import Any

from pydantic import BaseModel, field_validator


class Address(BaseModel):
    building_id: str | None = None
    city: str | None = None
    country: str | None = None
    country_id: str | None = None
    floor: str | None = None
    full_address: str
    full_street: str | None = None
    match_end: int | str | None = None
    match_start: int | str | None = None
    occupancy: str | None = None
    postal_code: str | None = None
    region1: str | None = None
    route_id: str | None = None
    state: str | None = None
    street: str | None = None
    street_name: str | None = None
    street_number: str | None = None
    street_type: str | None = None

    @field_validator("*", mode="before")
    @classmethod
    def strip_chars(cls, v: Any) -> Any:
        if isinstance(v, str):
            return v.strip(" ,;:")
        if v:
            return v

    def __str__(self) -> str:
        # Address object is represented as textual address
        address = ""
        try:
            address = self.full_address
        except AttributeError:
            pass

        return address



"""
pyap.source_US.data
~~~~~~~~~~~~~~~~~~~~

This module provides regular expression definitions required for
detecting Canada addresses.

The module is expected to always contain 'full_address' variable containing
all address parsing definitions.

:copyright: (c) 2015 by Vladimir Goncharov.
:license: MIT, see LICENSE for more details.
"""

import re

"""
Numerals from one to nine
Note: here and below we use syntax like '[Oo][Nn][Ee]'
instead of '(one)(?i)' to match 'One' or 'oNe' because
Python Regexps don't seem to support turning On/Off
case modes for subcapturing groups.
"""
zero_to_nine = r"""(?:
    [Zz][Ee][Rr][Oo]\s|[Oo][Nn][Ee]\s|[Tt][Ww][Oo]\s|
    [Tt][Hh][Rr][Ee][Ee]\s|[Ff][Oo][Uu][Rr]\s|
    [Ff][Ii][Vv][Ee]\s|[Ss][Ii][Xx]\s|
    [Ss][Ee][Vv][Ee][Nn]\s|[Ee][Ii][Gg][Hh][Tt]\s|
    [Nn][Ii][Nn][Ee]\s|[Tt][Ee][Nn]\s|
    [Ee][Ll][Ee][Vv][Ee][Nn]\s|
    [Tt][Ww][Ee][Ll][Vv][Ee]\s|
    [Tt][Hh][Ii][Rr][Tt][Ee][Ee][Nn]\s|
    [Ff][Oo][Uu][Rr][Tt][Ee][Ee][Nn]\s|
    [Ff][Ii][Ff][Tt][Ee][Ee][Nn]\s|
    [Ss][Ii][Xx][Tt][Ee][Ee][Nn]\s|
    [Ss][Ee][Vv][Ee][Nn][Tt][Ee][Ee][Nn]\s|
    [Ee][Ii][Gg][Hh][Tt][Ee][Ee][Nn]\s|
    [Nn][Ii][Nn][Ee][Tt][Ee][Ee][Nn]\s
    )
"""

# Numerals - 10, 20, 30 ... 90
ten_to_ninety = r"""(?:
    [Tt][Ee][Nn]\s|[Tt][Ww][Ee][Nn][Tt][Yy]\s|
    [Tt][Hh][Ii][Rr][Tt][Yy]\s|
    [Ff][Oo][Rr][Tt][Yy]\s|
    [Ff][Oo][Uu][Rr][Tt][Yy]\s|
    [Ff][Ii][Ff][Tt][Yy]\s|[Ss][Ii][Xx][Tt][Yy]\s|
    [Ss][Ee][Vv][Ee][Nn][Tt][Yy]\s|
    [Ee][Ii][Gg][Hh][Tt][Yy]\s|
    [Nn][Ii][Nn][Ee][Tt][Yy]\s
    )"""

# One hundred
hundred = r"""(?:
    [Hh][Uu][Nn][Dd][Rr][Ee][Dd]\s
    )"""

# One thousand
thousand = r"""(?:
    [Tt][Hh][Oo][Uu][Ss][Aa][Nn][Dd]\s
    )"""

"""
Regexp for matching street number.
Street number can be written 2 ways:
1) Using letters - "One thousand twenty two"
2) Using numbers
   a) - "1022"
   b) - "85-1190"
        "85 - 1190"
        "85th - 1190"
   c) - "85 1190"
"""
street_number = r"""
(?<![\.0-9])(?P<street_number>
    (?:
        [Aa][Nn][Dd]\s
        |
        {thousand}
        |
        {hundred}
        |
        {zero_to_nine}
        |
        {ten_to_ninety}
    ){from_to}
    |
    # 85th - 1190
    (?:\d{from_to}(?:th)?
        (?:\s?\-?\s?\d{from_to}(?:th)?)?\s
    )
    |
    # 45
    (?:\d{from_to}(?=[\s,]))
)
""".format(
    thousand=thousand,
    hundred=hundred,
    zero_to_nine=zero_to_nine,
    ten_to_ninety=ten_to_ninety,
    from_to="{1,5}",
)

"""
Regexp for matching street name.
In example below:
"Hoover Boulevard": "Hoover" is a street name
"""
street_name = r"(?P<street_name>\w[\w0-9\'\-\s\.]{0,30}?)"

post_direction = r"""
(?P<post_direction>
    (?:
        # English
        [Nn][Oo][Rr][Tt][Hh]{d}|
        [Ss][Oo][Uu][Tt][Hh]{d}|
        [Ee][Aa][Ss][Tt]{d}|
        [Ww][Ee][Ss][Tt]{d}|
        [Nn][Oo][Rr][Tt][Hh][Ee][Aa][Ss][Tt]{d}|
        [Nn][Oo][Rr][Tt][Hh][Ww][Ee][Ss][Tt]{d}|
        [Ss][Oo][Uu][Tt][Hh][Ee][Aa][Ss][Tt]{d}|
        [Ss][Oo][Uu][Tt][Hh][Ww][Ee][Ss][Tt]{d}|
        # French
        [Ee][Ss][Tt]{d}|
        [Nn][Oo][Rr][Dd]{d}|
        [Nn][Oo][Rr][Dd]\-[Ee][Ss][Tt]{d}|
        [Nn][Oo][Rr][Dd]\-[Oo][Uu][Ee][Ss][Tt]{d}|
        [Ss][Uu][Dd]{d}|
        [Ss][Uu][Dd]\-[Ee][Ss][Tt]{d}|
        [Ss][Uu][Dd]\-[Oo][Uu][Ee][Ss][Tt]{d}|
        [Oo][Uu][Ee][Ss][Tt]{d}
    )
    |
    (?:
        # English
        NW{d}|NE{d}|SW{d}|SE{d}|
        # French (missing above)
        NO{d}|SO{d}
    )
    |
    (?:
        # English
        N\.?\s?|S\.?\s?|E\.?\s?|W\.?\s?|
        # French (missing above)
        O\.?\s?
    )
)
""".format(d=r"[\s,]")

# Regexp for matching street type
# According to
# https://www.canadapost.ca/tools/pg/manual/PGaddress-e.asp#1385939
street_type = r"""
(?P<street_type>
    [Aa][Bb][Bb][Ee][Yy]{div}|
    [Aa][Cc][Rr][Ee][Ss]{div}|
    [Aa][Ll][Ll][Éé][Ee]{div}|
    [Aa][Ll][Ll][Ee][Yy]{div}|
    [Aa][Uu][Tt][Oo][Rr][Oo][Uu][Tt][Ee]{div}|[Aa][Uu][Tt]{div}|
    [Aa][Vv][Ee][Nn][Uu][Ee]{div}|[Aa][Vv][Ee]?{div}|
    [Bb][Aa][Yy]{div}|
    [Bb][Ee][Aa][Cc][Hh]{div}|
    [Bb][Ee][Nn][Dd]{div}|
    [Bb][Oo][Uu][Ll][Ee][Vv][Aa][Er][Dd]{div}|[Bb][Ll][Vv][Dd]{div}|[Bb][Oo][Uu][Ll]{div}|
    # Broadway
    [Bb][Rr][Oo][Aa][Dd][Ww][Aa][Yy]{div}|
    [Bb][Yy]\-?[Pp][Aa][Ss][Ss]{div}|
    [Bb][Yy][Ww][Aa][Yy]{div}|
    [Cc][Aa][Mm][Pp][Uu][Ss]{div}|
    [Cc][Aa][Pp][Ee]{div}|
    [Cc][Aa][Rr][Rr][EéÉ]{div}|[Cc][Aa][Rr]{div}|
    [Cc][Aa][Rr][Rr][Ee][Ff][Oo][Uu][Rr]{div}|[Cc][Aa][Rr][Re][Ee][Ff]{div}|
    [Cc][Ee][Nn][Tt][Rr][Ee]{div}|[Cc][Tt][Rr]{div}|
    [Cc][Ee][Rr][Cc][Ll][Ee]{div}|
    [Cc][Hh][Aa][Ss][Ee]{div}|
    [Cc][Hh][Ee][Mm][Ii][Nn]{div}|[Cc][Hh]{div}|
    [Cc][Ii][Rr][Cc][Ll][Ee]{div}|[Cc][Ii][Rr]{div}|
    [Cc][Ii][Rr][Cc][Uu][Ii][Tt]{div}|[Cc][Ii][Rr][Cc][Tt]{div}|
    [Cc][Ll][Oo][Ss][Ee]{div}|
    [Cc][Oo][Mm][Mm][Oo][Nn]{div}|
    [Cc][Oo][Nn][Cc][Ee][Ss][Ss][Ii][Oo][Nn]{div}|[Cc][Oo][Nn][Cc]{div}|
    [Cc][Oo][Rr][Nn][Ee][Rr][Ss]{div}|
    [Cc][Ôô][Tt][Ee]{div}|
    [Cc][Oo][Uu][Rr][Ss]{div}|
    [Cc][Oo][Uu][Rr]{div}|
    [Cc][Oo][Uu][Rr][Tt]{div}|[Cc][Rr][Tt]{div}|
    [Cc][Oo][Vv][Ee]{div}|
    [Cc][Rr][Ee][Ss][Cc][Ee][Nn][Tt]{div}|[Cc][Rr][Ee][Ss]{div}|
    [Cc][Rr][Oo][Ii][Ss][Ss][Aa][Nn][Tt]{div}|[Cc][Rr][Oo][Ii][Ss]{div}|
    [Cc][Rr][Oo][Ss][Ss][Ii][Nn][Gg]{div}|[Cc][Rr][Oo][Ss][Ss]{div}|
    [Cc][Uu][Ll]\-[Dd][Ee]\-[Ss][Aa][Cc]{div}|[Cc][Dd][Ss]{div}|
    [Dd][Aa][Ll][Ee]{div}|
    [Dd][Ee][Ll][Ll]{div}|
    [Dd][Ii][Vv][Ee][Rr][Ss][Ii][Oo][Nn]{div}|[Dd][Ii][Vv][Ee][Rr][Ss]{div}|
    [Dd][Oo][Ww][Nn][Ss]{div}|
    [Dd][Rr][Ii][Vv][Ee]{div}|[Dd][Rr]{div}|
    [Ée][Cc][Hh][Aa][Nn][Gg][Ee][Uu][Rr]{div}|[Ée][Cc][Hh]{div}|
    [Ee][Nn][Dd]{div}|
    [Ee][Ss][Pp][Ll][Aa][Nn][Aa][Dd][Ee]{div}|[Ee][Ss][Pp][Ll]{div}|
    [Ee][Ss][Tt][Aa][Tt][Ee][Ss]?{div}|
    [Ee][Xx][Pp][Rr][Ee][Ss][Ss][Ww][Aa][Yy]{div}|[Ee][Xx][Pp][Yy]{div}|
    [Ee][Xx][Tt][Ee][Nn][Ss][Ii][Oo][Nn]{div}|[Ee][Xx][Tt][Ee][Nn]{div}|
    [Ff][Aa][Rr][Mm]{div}|
    [Ff][Ii][Ee][Ll][Dd]{div}|
    [Ff][Oo][Rr][Ee][Ss][Tt]{div}|
    [Ff][Rr][Ee][Ee][Ww][Aa][Yy]{div}|[Ff][Ww][Yy]{div}|
    [Ff][Rr][Oo][Nn][Tt]{div}|
    [Gg][Aa][Rr][Dd][Ee][Nn][Ss]{div}|[Gg][Dd][Nn][Ss]{div}|
    [Gg][Aa][Tt][Ee]{div}|
    [Gg][Ll][Aa][Dd][Ee]{div}|
    [Gg][Ll][Ee][Nn]{div}|
    [Gg][Rr][Ee][Ee][Nn]{div}|
    [Gg][Rr][Uo][Uu][Nn][Dd][Ss]{div}|[Gg][Rr][Nn][Dd][Ss]{div}|
    [Gg][Rr][Oo][Vv][Ee]{div}|
    [Hh][Aa][Rr][Bb][Oo][Uu][Rr]{div}|[Hh][Aa][Rr][Bb][Rr]{div}|
    [Hh][Ee][Aa][Tt][Hh]{div}|
    [Hh][Ee][Ii][Gg][Hh][Tt][Ss]{div}|[Hh][Tt][Ss]{div}|
    [Hh][Ii][Gg][Hh][Ll][Aa][Nn][Dd][Ss]{div}|[Hh][Gg][Hh][Ll][Dd][Sd]{div}|
    [Hh][Ii][Gg][Gh][Ww][Aa][Yy]{div}|[Hh][Ww][Yy]{div}|
    [Hh][Ii][Ll][Ll]{div}|
    [Hh][Oo][Ll][Ll][Oo][Ww]{div}|
    [Îi][Ll][Ee]{div}|
    [Ii][Mm][Pp][Aa][Ss][Ss][Ee]{div}|I[Mm][Pp]{div}|
    [Ii][Nn][Ll][Ee][Tt]{div}|
    [Ii][Ss][Ll][Aa][Nn][Dd]{div}|
    [Kk][Ee][Yy]{div}|
    [Kk][Nn][Oo][Ll][Ll]{div}|
    [Ll][Aa][Nn][Dd][Ii][Nn][Gg]{div}|[Ll][Aa][Nn][Dd][Nn][Gg]{div}|
    [Ll][Aa][Nn][Ee]{div}|
    [Ll][Ii][Mm][Ii][Tt][Ss]{div}|[Ll][Mm][Tt][Ss]{div}|
    [Ll][Ii][Nn][Ee]{div}|
    [Ll][Ii][Nn][Kk]{div}|
    [Ll][Oo][Oo][Kk][Oo][Uu][Tt]{div}|[Ll][Kk][Oo][Uu][Tt]{div}|
    [Mm][Aa][Ii][Nn][Ww][Aa][Yy]{div}|
    [Mm][Aa][Ll][Ll]{div}|
    [Mm][Aa][Nn][Oo][Rr]{div}|
    [Mm][Aa][Zz][Ee]{div}|
    [Mm][Ee][Aa][Dd][Oo][Ww]{div}|
    [Mm][Ee][Ww][Ss]{div}|
    [Mm][Oo][Nn][Tt][Éé][Ee]{div}|
    [Mm][Oo][Oo][Rr]{div}|
    [Mm][Oo][Uu][Nn][Tt][Aa][Ii][Nn]{div}|[Mm][Tt][Nn]{div}|
    [Mm][Oo][Uu][Nn][Tt]{div}|
    [Oo][Rr][Cc][Hh][Aa][Rr][Dd]{div}|[Oo][Rr][Cc][Hh]{div}|
    [Pp][Aa][Rr][Aa][Dd][Ee]{div}|
    [Pp][Aa][Rr][Cc]{div}|
    [Pp][Aa][Rr][Kk][Ww][Aa][Yy]{div}|[Pp][Kk][Yy]{div}|
    [Pp][Aa][Rr][Kk]{div}|[Pp][Kk]{div}|
    [Pp][Aa][Ss][Ss][Aa][Gg][Ee]{div}|[Pp][As][Ss][Ss]{div}|
    [Pp][Aa][Tt][Hh]{div}|
    [Pp][Aa][Tt][Hh][Ww][Aa][Yy]{div}|[Pp][Tt][Ww][Aa][Yy]{div}|
    [Pp][Ii][Nn][Ee][Ss]{div}|
    [Pp][Ll][Aa][Cc][Ee]{div}|[Pp][Ll]{div}|
    [Pp][Ll][Aa][Tt][Ee][Aa][Uu]{div}|[Pp][Ll][Aa][Tt]{div}|
    [Pp][Ll][Aa][Zz][Aa]{div}|
    [Pp][Oo][Ii][Nn][Tt][Ee]{div}|
    [Pp][Oo][Ii][Nn][Tt]{div}|[Pp][Tt]{div}|
    [Pp][Oo][Rr][Tt]{div}|
    [Pp][Rr][Ii][Vv][Aa][Tt][Ee]{div}|[Pp][Vv][Tt]{div}|
    [Pp][Rr][Oo][Mm][Ee][Nn][Aa][Dd][Ee]{div}|[Pp][Rr][Oo][Mm]{div}|
    [Qq][Uu][Aa][Ii]{div}|
    [Qq][Uu][Aa][Yy]{div}|
    [Rr][Aa][Mm][Pp]{div}|
    [Rr][Aa][Nn][Gg][Ee]{div}|[Rr][Gg]{div}|
    [Rr][Aa][Nn][Gg]{div}|
    [Rr][Ii][Dd][Gg][Ee]{div}|
    [Rr][Ii][Ss][Ee]{div}|
    [Rr][Oo][Aa][Dd]{div}|[Rr][Dd]{div}|
    [Rr][Oo][Nn][Dd]\-[Pp][Oo][Ii][Nn][Tt]{div}|[Rr][Dd][Pp][Tt]{div}|
    [Rr][Oo][Uu][Tt][Ee]{div}|[Rr][Tt][Ee]{div}|
    [Rr][Oo][Ww]{div}|
    [Rr][Uu][Ee][Ll][Ll][Ee]{div}|[Rr][Ll][Ee]{div}|
    [Rr][Uu][Ee]{div}|
    [Rr][Uu][Nn]{div}|
    [Ss][Ee][Nn][Tt][Ii][Ee][Rr]{div}|[Ss][Ee][Nn][Tt]{div}|
    # Street
    [Ss][Tt][Rr][Ee][Ee][Tt]{div}|[Ss][Tt](?![A-Za-z]){div}|
    # Square
    [Ss][Qq][Uu][Aa][Rr][Ee]{div}|[Ss][Qq]{div}|
    [Ss][Uu][Bb][Dd][Ii][Vv][Ii][Ss][Ii][Oo][Nn]{div}|[Ss][Uu][Bb][Dd][Ii][Vv]{div}|
    [Tt][Ee][Rr][Rr][Aa][Cc][Ee]{div}|[Tt][Ee][Re][Re]{div}|
    [Tt][Ee][Rr][Rr][Aa][Ss][Ss][Ee]{div}|[Tt][Ss][Ss][Es]{div}|
    [Tt][Hh][Ii][Cc][Kk][Ee][Tt]{div}|[Tt][Hh][Ii][Cc][Kk]{div}|
    [Tt][Oo][Ww][Ee][Rr][Ss]{div}|
    [Tt][Oo][Ww][Nn][Ll][Ii][Nn][Ee]{div}|[Tt][Ll][Ii][Nn][Ee]{div}|
    [Tt][Rr][Aa][Ii][Ll]{div}|
    [Tt][Uu][Rr][Nn][Aa][Bb][Oo][Uu][Tt]{div}|[Tt][Rr][Nn][Aa][Bb][Tt]{div}|
    [Vv][Aa][Ll][Ee]{div}|
    [Vv][Ii][Aa]{div}|
    [Vv][Ii][Ee][Ww]{div}|
    [Vv][Ii][Ll][Ll][Aa][Gg][Ee]{div}|[Vv][Ii][Ll][Ll][Gg][Ee]{div}|
    [Vv][Ii][Ll][Ll][Aa][Ss]{div}|
    [Vv][Ii][Ss][Tt][Aa]{div}|
    [Vv][Oo][Ii][Ee]{div}|
    [Ww][Aa][Ll][Lk]{div}|
    [Ww][Aa][Yy]{div}|
    [Ww][Hh][Aa][Rr][Ff]{div}|
    [Ww][Oo][Oo][Dd]{div}|
    [Ww][Yy][Nn][Dd]{div}
)
(?P<route_id>
    [\(\s\,]{route_symbols}
    [Rr][Oo][Uu][Tt][Ee]\s[A-Za-z0-9]+[\)\s\,]{route_symbols}
)?
""".format(div=r"[\.\s,]{0,2}", route_symbols="{0,3}")

floor = r"""
(?P<floor>
    (?:
    \d+[A-Za-z]{0,2}\.?\s[Ff][Ll][Oo][Oo][Rr]\s
    )
    |
    (?:
        [Ff][Ll][Oo][Oo][Rr]\s\d+[A-Za-z]{0,2}\s
    )
)
"""

building = rf"""
(?P<building_id>
    (?:
        (?:[Bb][Uu][Ii][Ll][Dd][Ii][Nn][Gg])
        |
        (?:[Bb][Ll][Dd][Gg])
    )
    \s
    (?:
        (?:
            [Aa][Nn][Dd]\s
            |
            {thousand}
            |
            {hundred}
            |
            {zero_to_nine}
            |
            {ten_to_ninety}
        ){{1,5}}
        |
        \d{{0,4}}[A-Za-z]?
    )
    \s ?
)  # end building_id
"""

occupancy = r"""
(?:
    (?:
        (?:
            #
            # English
            #
            # Suite
            [Ss][Uu][Ii][Tt][Ee]\s|[Ss][Tt][Ee]\.?\s
            |
            # Apartment
            [Aa][Pp][Tt]\.?\s|[Aa][Pp][Aa][Rr][Tt][Mm][Ee][Nn][Tt]\s
            |
            # Room
            [Rr][Oo][Oo][Mm]\s|[Rr][Mm]\.?\s
            |
            # Unit
            [Uu][Nn][Ii][Tt]\s
            |
            #
            # French
            #
            # Apartement
            [Aa][Pp][Aa][Rr][Tt][Ee][Mm][Ee][Nn][Tt]\s|A[Pp][Pp]\s
            |
            # Bureau
            [Bb][Uu][Rr][Ee][Aa][Uu]\s
            |
            # Unité
            [Uu][Nn][Ii][Tt][Éé]\s
        )
        (?:
            [A-Za-z\#\&\-\d]{1,7}
        )?
    )
    |
    (?:
        \#[0-9]{,3}[A-Za-z]{1}
    )
)\s?
"""

po_box = r"""
    (?P<postal_box>
        # English - PO Box 123
        (?:[Pp]\.?\s?[Oo]\.?\s[Bb][Oo][Xx]\s\d+)
        |
        # French - B.P. 123
        (?:[Bb]\.?\s[Pp]\.?\s\d+)
        |
        # C.P. 123
        (?:[Cc]\.?\s[Pp]\.?\s\d+)
        |
        # Case postale 123
        (?:[Cc]ase\s[Pp][Oo][Ss][Tt][Aa][Ll][Ee]\s\d+)
        |
        # C.P. 123
        (?:[Cc]\.[Pp]\.\s\d+)
    )
"""

"""
Define detection rules for a second type of address format (the French one)
"""
street_number_b = re.sub(r"<([a-z\s]+)>", r"<\1_b>", street_number)
street_name_b = re.sub(r"<([a-z\s]+)>", r"<\1_b>", street_name)
street_type_b = re.sub(r"<([a-z\s]+)>", r"<\1_b>", street_type)
po_box_b = re.sub(r"<([a-z\s]+)>", r"<\1_b>", po_box)
post_direction_b = re.sub(r"<([a-z\s*]+)>", r"<\1_b>", post_direction)

po_box_positive_lookahead = r"""
    (?=
        # English - PO Box 123
        (?:[Pp]\.?\s?[Oo]\.?\s[Bb][Oo][Xx]\s\d+)
        |
        # French - B.P. 123
        (?:[Bb]\.?\s[Pp]\.?\s\d+)
        |
        # C.P. 123
        (?:[Cc]\.?\s[Pp]\.?\s\d+)
        |
        # Case postale 123
        (?:[Cc]ase\s[Pp][Oo][Ss][Tt][Aa][Ll][Ee]\s\d+)
        |
        # C.P. 123
        (?:[Cc]\.[Pp]\.\s\d+)
        |
        (?:[\s\,])
    )
"""

full_street = r"""
(?:
    # Format commonly used in French
    (?P<full_street_b>

        {street_number_b}{div}
        {street_type_b}{div}
        ({street_name_b} {po_box_positive_lookahead})?\,?\s?
        {post_direction_b}?\,?\s?
        {po_box_b}?\,?\s?
    )
    |
    # Format commonly used in English
    (?P<full_street>

        {street_number}\,?\s?
        {street_name}?\,?\s?
        (?:(?<=[\s\,]){street_type})\,?\s?
        {post_direction}?\,?\s?
        {floor}?\,?\s?

        (?P<building_id>
            {building}
        )?\,?\s?

        (?P<occupancy>
            {occupancy}
        )?\,?\s?

        {po_box}?
    )
)""".format(
    street_number=street_number,
    street_number_b=street_number_b,
    street_name=street_name,
    street_name_b=street_name_b,
    street_type=street_type,
    street_type_b=street_type_b,
    post_direction=post_direction,
    post_direction_b=post_direction_b,
    floor=floor,
    building=building,
    occupancy=occupancy,
    po_box=po_box,
    po_box_b=po_box_b,
    po_box_positive_lookahead=po_box_positive_lookahead,
    div=r"[\s,]{1,2}",
)

# region1 here is actually a "province"
region1 = r"""
(?P<region1>
    (?:
        # province abbreviations (English)
        A\.?B\.?|B\.?C\.?|M\.?B\.?|N\.?B\.?|N\.?L\.?|
        N\.?T\.?|N\.?S\.?|N\.?U\.?|O\.?N\.?|P\.?E\.?|
        Q\.?C\.?|S\.?K\.?|Y\.?T\.?
    )
    |
    (?:
        # provinces full (English)
        [Aa][Ll][Bb][Ee][Rr][Tt][Aa]|
        [Bb][Rr][Ii][Tt][Ii][Ss][Hh]\s[Cc][Oo][Ll][Uu][Mm][Bb][Ii][Aa]|
        [Mm][Aa][Nn][Ii][Tt][Oo][Bb][Aa]|
        [Nn][Ee][Ww]\s[Bb][Rr][Uu][Nn][Ss][Ww][Ii][Cc][Kk]|
        [Nn][Ee][Ww][Ff][Oo][Uu][Nn][Dd][Ll][Aa][Nn][Dd]\s
        [Aa][Nn][Dd]\s[Ll][Aa][Bb][Rr][Aa][Dd][Oo][Rr]|
        [Nn][Ee][Ww][Ff][Oo][Uu][Nn][Dd][Ll][Aa][Nn][Dd]\s
        \&\s[Ll][Aa][Bb][Rr][Aa][Dd][Oo][Rr]|
        [Nn][Oo][Rr][Tt][Hh][Ww][Ee][Ss][Tt]\s
        [Tt][Ee][Rr][Rr][Ii][Tt][Oo][Rr][Ii][Ee][Ss]|
        [Nn][Oo][Vv][Aa]\s[Ss][Cc][Oo][Tt][Ii][Aa]|
        [Nn][Uu][Nn][Aa][Vv][Uu][Tt]|
        [Oo][Nn][Tt][Aa][Rr][Ii][Oo]|
        [Pp][Rr][Ii][Nn][Cc][Ee]\s[Ee][Dd][Ww][Aa][Rr][Dd]\s
        [Ii][Ss][Ll][Aa][Nn][Dd]|
        [Qq][Uu][Ee][Bb][Ee][Cc]|
        [Ss][Aa][Ss][Kk][Aa][Tt][Cc][Hh][Ee][Ww][Aa][Nn]|
        [Yy][Uu][Kk][Oo][Nn]|
        # provinces full (French)
        [Cc][Oo][Ll][Oo][Mm][Bb][Ii][Ee]\-
        [Bb][Rr][Ii][Tt][Aa][Nn]{1,2}[Ii][Qq][Eu][Ee]|
        [Nn][Oo][Uu][Vv][Ee][Aa][Uu]\-[Bb][Rr][Uu][Nn][Ss][Ww][Ii][Cc][Kk]|
        [Tt][Ee][Rr][Rr][Ee]\-[Nn][Ee][Uu][Vv][Ee]\-
        [Ee][Tt]\-[Ll][Aa][Bb][Rr][Aa][Dd][Oo][Rr]|
        [Tt][Ee][Rr][Rr][Ii][Tt][Oo][Ii][Rr][Ee][Ss]\s[Dd][Uu]\s
        [Nn][Oo][Rr][Dd]\-[Oo][Uu][Ee][Ss][Tt]|
        [Nn][Oo][Uu][Vv][Ee][Ll][Ll][Ee]\-[ÉéEe][Cc][Oo][Ss][Ss][Ee]|
        [ÎîIi][Ll][Ee]\-[Dd][Uu]\-[Pp][Rr][Ii][Nn][Cc][Ee]\-
        [ÉéEe][Dd][Oo][Uu][Aa][Rr][Dd]|
        [Qq][Uu][Éé][Bb][Ee][Cc]
    )
)
"""

city = r"""
(?P<city>
    (?<=[\, ])[A-z]{1}(?![0-9]) # city second char should not be number
    [\w\s\-\'\.]{2,20}?(?=[\, ])
)
"""

postal_code = r"""
(?P<postal_code>
    (?:
        [ABCEGHJKLMNPRSTVXY]\d[ABCEGHJKLMNPRSTVWXYZ]\s?
        \d[ABCEGHJKLMNPRSTVWXYZ]\d
    )
)
"""

country = r"""
            (?:
                [Cc][Aa][Nn][Aa][Dd][Aa]
            )
            """

# define detection rules for postal code placed in different parts of address
postal_code_b = re.sub(r"<([a-z\s]+)>", r"<\1_b>", postal_code)
postal_code_c = re.sub(r"<([a-z\s]+)>", r"<\1_c>", postal_code)

full_address = r"""
                (?P<full_address>
                    {full_street} {div}
                    {city} {div}
                    (?:{postal_code_c} {div})?
                    \(?{region1}[\)\.]? {div}
                    (?:
                        (?:
                            {postal_code}? {div} {country}? 
                            (?:{div} {postal_code_b})?
                        )
                    )
                )
                """.format(
    full_street=full_street,
    div=r"[\, ]{,2}",
    city=city,
    region1=region1,
    country=country,
    postal_code=postal_code,
    postal_code_b=postal_code_b,
    postal_code_c=postal_code_c,
)



"""
pyap.parser
~~~~~~~~~~~~~~~~

This module contains AddressParser class which connects all the
functionality of the package in one place.

:copyright: (c) 2015 by Vladimir Goncharov.
:license: MIT, see LICENSE for more details.
"""

import importlib
import re
from typing import Any

from . import address, exceptions as e, utils


class AddressParser:
    country: str
    clean_text: str

    def __init__(self, **kwargs: Any):
        """Initialize with custom arguments"""

        for k, v in kwargs.items():
            # store country id in uppercase
            if k == "country":
                v = v.upper()
            setattr(self, k, v)
        try:
            # import detection rules
            package = "pyap_beauhurst" + ".source_" + self.country + ".data"
            data = importlib.import_module(package)
            self.rules = data.full_address

        except AttributeError:
            raise e.NoCountrySelected(
                "No country specified during library initialization.", "Error 1"
            ) from None

        except ImportError:
            raise e.CountryDetectionMissing(
                f'Detection rules for country "{self.country}" not found.',
                "Error 2",
            ) from None

    def parse(self, text: str) -> list[address.Address | None]:
        """
        Returns a list of addresses found in text together with parsed address parts
        """
        results: list[address.Address | None] = []
        self.clean_text = self._normalize_string(text)

        # get addresses
        address_matches = utils.finditer(self.rules, self.clean_text)

        if address_matches:
            # append parsed address info
            results: list[address.Address | None] = list(  # type: ignore[no-redef]
                map(self._parse_address, address_matches)
            )

        return results

    def _parse_address(self, match: re.Match | str) -> address.Address | None:
        """Parses address into parts"""
        if isinstance(match, str):
            # If the address is passed as a match it saves doing the match twice
            match: re.Match | None = utils.match(  # type: ignore[no-redef]
                self.rules, match, flags=re.VERBOSE | re.U
            )
        if match:
            if isinstance(match, re.Match):
                match_as_dict = match.groupdict()
                match_as_dict.update({"country_id": self.country})
                # combine results
                cleaned_dict = self._combine_results(match_as_dict)
                cleaned_dict["match_start"] = match.start()
                cleaned_dict["match_end"] = match.end()
                # create object containing results
                return address.Address.model_validate(cleaned_dict)

        return None

    @staticmethod
    def _combine_results(
        match_as_dict: dict[str, str | int | None],
    ) -> dict[str, str | int | None]:
        """Combine results from different parsed parts:
        we look for non-empty results in values like
        'postal_code_b' or 'postal_code_c' and store
        them as main value.

        So 'postal_code_b':'123456'
            becomes:
           'postal_code'  :'123456'
        """
        keys: list[str] = []
        vals: list[str | int | None] = []
        for k, v in match_as_dict.items():
            if k[-2:] in "_a_b_c_d_e_f_g_h_i_j_k_l_m":
                if v:
                    # strip last 2 chars: '..._b' -> '...'
                    keys.append(k[:-2])
                    vals.append(v)
            else:
                if k not in keys:
                    keys.append(k)
                    vals.append(v)
        return dict(zip(keys, vals, strict=True))

    @staticmethod
    def _normalize_string(text: str) -> str:
        """Prepares incoming text for parsing:
        removes excessive spaces, tabs, newlines, etc.
        """
        conversion = {
            # newlines
            r"\r*(\n\r*)+": ", ",
            r"\s*(\,\s*)+": ", ",
            # replace excessive empty spaces
            r"\s+": " ",
            # convert all types of hyphens/dashes to a
            # simple old-school dash
            # from http://utf8-chartable.de/unicode-utf8-table.pl?
            # start=8192&number=128&utf8=string-literal
            "‐": "-",
            "‑": "-",
            "‒": "-",
            "–": "-",
            "—": "-",
            "―": "-",
        }
        for find, replace in conversion.items():
            text = re.sub(find, replace, text, flags=re.UNICODE)
        return text



"""
pyap.source_US.data
~~~~~~~~~~~~~~~~~~~~

This module provides regular expression definitions required for
detecting US addresses.

The module is expected to always contain 'full_address' variable containing
all address parsing definitions.

:copyright: (c) 2015 by Vladimir Goncharov.
:license: MIT, see LICENSE for more details.
"""

import string

"""Numerals from one to nine
Note: here and below we use syntax like '[Oo][Nn][Ee]'
instead of '(one)(?i)' to match 'One' or 'oNe' because
Python Regexps don't seem to support turning On/Off
case modes for subcapturing groups.
"""
zero_to_nine = r"""(?:
    [Zz][Ee][Rr][Oo]\s|[Oo][Nn][Ee]\s|[Tt][Ww][Oo]\s|
    [Tt][Hh][Rr][Ee][Ee]\s|[Ff][Oo][Uu][Rr]\s|
    [Ff][Ii][Vv][Ee]\s|[Ss][Ii][Xx]\s|
    [Ss][Ee][Vv][Ee][Nn]\s|[Ee][Ii][Gg][Hh][Tt]\s|
    [Nn][Ii][Nn][Ee]\s|[Tt][Ee][Nn]\s|
    [Ee][Ll][Ee][Vv][Ee][Nn]\s|
    [Tt][Ww][Ee][Ll][Vv][Ee]\s|
    [Tt][Hh][Ii][Rr][Tt][Ee][Ee][Nn]\s|
    [Ff][Oo][Uu][Rr][Tt][Ee][Ee][Nn]\s|
    [Ff][Ii][Ff][Tt][Ee][Ee][Nn]\s|
    [Ss][Ii][Xx][Tt][Ee][Ee][Nn]\s|
    [Ss][Ee][Vv][Ee][Nn][Tt][Ee][Ee][Nn]\s|
    [Ee][Ii][Gg][Hh][Tt][Ee][Ee][Nn]\s|
    [Nn][Ii][Nn][Ee][Tt][Ee][Ee][Nn]\s
    )"""

# Numerals - 10, 20, 30 ... 90
ten_to_ninety = r"""(?:
    [Tt][Ee][Nn]\s|[Tt][Ww][Ee][Nn][Tt][Yy]\s|
    [Tt][Hh][Ii][Rr][Tt][Yy]\s|
    [Ff][Oo][Rr][Tt][Yy]\s|
    [Ff][Oo][Uu][Rr][Tt][Yy]\s|
    [Ff][Ii][Ff][Tt][Yy]\s|[Ss][Ii][Xx][Tt][Yy]\s|
    [Ss][Ee][Vv][Ee][Nn][Tt][Yy]\s|
    [Ee][Ii][Gg][Hh][Tt][Yy]\s|
    [Nn][Ii][Nn][Ee][Tt][Yy]\s
    )"""

# One hundred
hundred = r"""(?:
    [Hh][Uu][Nn][Dd][Rr][Ee][Dd]\s
    )"""

# One thousand
thousand = r"""(?:
    [Tt][Hh][Oo][Uu][Ss][Aa][Nn][Dd]\s
    )"""

"""
Regexp for matching street number.
Street number can be written 2 ways:
1) Using letters - "One thousand twenty two"
2) Using numbers
   a) - "1022"
   b) - "85-1190"
   c) - "85 1190"
"""
street_number = r"""(?P<street_number>
                        (?:
                            [Aa][Nn][Dd]\s
                            |
                            {thousand}
                            |
                            {hundred}
                            |
                            {zero_to_nine}
                            |
                            {ten_to_ninety}
                        ){from_to}
                        |
                        (?:\d{from_to}
                            (?:\s?\-?\s?\d{from_to})?\s
                        )
                    )
                """.format(
    thousand=thousand,
    hundred=hundred,
    zero_to_nine=zero_to_nine,
    ten_to_ninety=ten_to_ninety,
    from_to="{1,5}",
)

"""
Regexp for matching street name.
In example below:
"Hoover Boulevard": "Hoover" is a street name
"""
street_name = r"""(?P<street_name>
                  [a-zA-Z0-9\s\.]{3,31}  # Seems like the longest US street is
                                         # 'Northeast Kentucky Industrial Parkway'
                                         # https://atkinsbookshelf.wordpress.com/tag/longest-street-name-in-us/
                 )
              """

post_direction = r"""
                    (?P<post_direction>
                        (?:
                            [Nn][Oo][Rr][Tt][Hh]\s|
                            [Ss][Oo][Uu][Tt][Hh]\s|
                            [Ee][Aa][Ss][Tt]\s|
                            [Ww][Ee][Ss][Tt]\s
                        )
                        |
                        (?:
                            NW\s|NE\s|SW\s|SE\s
                        )
                        |
                        (?:
                            N\.?\s|S\.?\s|E\.?\s|W\.?\s
                        )
                    )
                """

# This list was taken from: https://pe.usps.com/text/pub28/28apc_002.htm
# Broadway and Lp (abbreviation for Loop) were added to the list
street_type_list = [
    "Allee",
    "Alley",
    "Ally",
    "Aly",
    "Anex",
    "Annex",
    "Annx",
    "Anx",
    "Arc",
    "Arcade",
    "Av",
    "Ave",
    "Aven",
    "Avenu",
    "Avenue",
    "Avn",
    "Avnue",
    "Bayoo",
    "Bayou",
    "Bch",
    "Beach",
    "Bend",
    "Bg",
    "Bgs",
    "Blf",
    "Blfs",
    "Bluf",
    "Bluff",
    "Bluffs",
    "Blvd",
    "Bnd",
    "Bot",
    "Bottm",
    "Bottom",
    "Boul",
    "Boulevard",
    "Boulv",
    "Br",
    "Branch",
    "Brdge",
    "Brg",
    "Bridge",
    "Brk",
    "Brks",
    "Brnch",
    "Broadway",
    "Brook",
    "Brooks",
    "Btm",
    "Burg",
    "Burgs",
    "Byp",
    "Bypa",
    "Bypas",
    "Bypass",
    "Byps",
    "Byu",
    "Camp",
    "Canyn",
    "Canyon",
    "Cape",
    "Causeway",
    "Causwa",
    "Cen",
    "Cent",
    "Center",
    "Centers",
    "Centr",
    "Centre",
    "Cir",
    "Circ",
    "Circl",
    "Circle",
    "Circles",
    "Cirs",
    "Clb",
    "Clf",
    "Clfs",
    "Cliff",
    "Cliffs",
    "Club",
    "Cmn",
    "Cmns",
    "Cmp",
    "Cnter",
    "Cntr",
    "Cnyn",
    "Common",
    "Commons",
    "Cor",
    "Corner",
    "Corners",
    "Cors",
    "Course",
    "Court",
    "Courts",
    "Cove",
    "Coves",
    "Cp",
    "Cpe",
    "Crcl",
    "Crcle",
    "Creek",
    "Cres",
    "Crescent",
    "Crest",
    "Crk",
    "Crossing",
    "Crossroad",
    "Crossroads",
    "Crse",
    "Crsent",
    "Crsnt",
    "Crssng",
    "Crst",
    "Cswy",
    "Ct",
    "Ctr",
    "Ctrs",
    "Cts",
    "Curv",
    "Curve",
    "Cv",
    "Cvs",
    "Cyn",
    "Dale",
    "Dam",
    "Div",
    "Divide",
    "Dl",
    "Dm",
    "Dr",
    "Driv",
    "Drive",
    "Drives",
    "Drs",
    "Drv",
    "Dv",
    "Dvd",
    "Est",
    "Estate",
    "Estates",
    "Ests",
    "Exp",
    "Expr",
    "Express",
    "Expressway",
    "Expw",
    "Expy",
    "Ext",
    "Extension",
    "Extensions",
    "Extn",
    "Extnsn",
    "Exts",
    "Fall",
    "Falls",
    "Ferry",
    "Field",
    "Fields",
    "Flat",
    "Flats",
    "Fld",
    "Flds",
    "Fls",
    "Flt",
    "Flts",
    "Ford",
    "Fords",
    "Forest",
    "Forests",
    "Forg",
    "Forge",
    "Forges",
    "Fork",
    "Forks",
    "Fort",
    "Frd",
    "Frds",
    "Freeway",
    "Freewy",
    "Frg",
    "Frgs",
    "Frk",
    "Frks",
    "Frry",
    "Frst",
    "Frt",
    "Frway",
    "Frwy",
    "Fry",
    "Ft",
    "Fwy",
    "Garden",
    "Gardens",
    "Gardn",
    "Gateway",
    "Gatewy",
    "Gatway",
    "Gdn",
    "Gdns",
    "Glen",
    "Glens",
    "Gln",
    "Glns",
    "Grden",
    "Grdn",
    "Grdns",
    "Green",
    "Greens",
    "Grn",
    "Grns",
    "Grov",
    "Grove",
    "Groves",
    "Grv",
    "Grvs",
    "Gtway",
    "Gtwy",
    "Harb",
    "Harbor",
    "Harbors",
    "Harbr",
    "Haven",
    "Hbr",
    "Hbrs",
    "Heights",
    "Highway",
    "Highwy",
    "Hill",
    "Hills",
    "Hiway",
    "Hiwy",
    "Hl",
    "Hllw",
    "Hls",
    "Hollow",
    "Hollows",
    "Holw",
    "Holws",
    "Hrbor",
    "Ht",
    "Hts",
    "Hvn",
    "Hway",
    "Hwy",
    "Inlet",
    "Inlt",
    "Is",
    "Island",
    "Islands",
    "Isle",
    "Isles",
    "Islnd",
    "Islnds",
    "Iss",
    "Jct",
    "Jction",
    "Jctn",
    "Jctns",
    "Jcts",
    "Junction",
    "Junctions",
    "Junctn",
    "Juncton",
    "Key",
    "Keys",
    "Knl",
    "Knls",
    "Knol",
    "Knoll",
    "Knolls",
    "Ky",
    "Kys",
    "Lake",
    "Lakes",
    "Land",
    "Landing",
    "Lane",
    "Lck",
    "Lcks",
    "Ldg",
    "Ldge",
    "Lf",
    "Lgt",
    "Lgts",
    "Light",
    "Lights",
    "Lk",
    "Lks",
    "Ln",
    "Lndg",
    "Lndng",
    "Loaf",
    "Lock",
    "Locks",
    "Lodg",
    "Lodge",
    "Loop",
    "Loops",
    "Lp",
    "Mall",
    "Manor",
    "Manors",
    "Mdw",
    "Mdws",
    "Meadow",
    "Meadows",
    "Medows",
    "Mews",
    "Mill",
    "Mills",
    "Mission",
    "Missn",
    "Ml",
    "Mls",
    "Mnr",
    "Mnrs",
    "Mnt",
    "Mntain",
    "Mntn",
    "Mntns",
    "Motorway",
    "Mount",
    "Mountain",
    "Mountains",
    "Mountin",
    "Msn",
    "Mssn",
    "Mt",
    "Mtin",
    "Mtn",
    "Mtns",
    "Mtwy",
    "Nck",
    "Neck",
    "Opas",
    "Orch",
    "Orchard",
    "Orchrd",
    "Oval",
    "Overpass",
    "Ovl",
    "Park",
    "Parks",
    "Parkway",
    "Parkways",
    "Parkwy",
    "Pass",
    "Passage",
    "Path",
    "Paths",
    "Pike",
    "Pikes",
    "Pine",
    "Pines",
    "Pkway",
    "Pkwy",
    "Pkwys",
    "Pky",
    "Pl",
    "Place",
    "Plain",
    "Plains",
    "Plaza",
    "Pln",
    "Plns",
    "Plz",
    "Plza",
    "Pne",
    "Pnes",
    "Point",
    "Points",
    "Port",
    "Ports",
    "Pr",
    "Prairie",
    "Prk",
    "Prr",
    "Prt",
    "Prts",
    "Psge",
    "Pt",
    "Pts",
    "Rad",
    "Radial",
    "Radiel",
    "Radl",
    "Ramp",
    "Ranch",
    "Ranches",
    "Rapid",
    "Rapids",
    "Rd",
    "Rdg",
    "Rdge",
    "Rdgs",
    "Rds",
    "Rest",
    "Ridge",
    "Ridges",
    "Riv",
    "River",
    "Rivr",
    "Rnch",
    "Rnchs",
    "Road",
    "Roads",
    "Route",
    "Row",
    "Rpd",
    "Rpds",
    "Rst",
    "Rte",
    "Rue",
    "Run",
    "Rvr",
    "Shl",
    "Shls",
    "Shoal",
    "Shoals",
    "Shoar",
    "Shoars",
    "Shore",
    "Shores",
    "Shr",
    "Shrs",
    "Skwy",
    "Skyway",
    "Smt",
    "Spg",
    "Spgs",
    "Spng",
    "Spngs",
    "Spring",
    "Springs",
    "Sprng",
    "Sprngs",
    "Spur",
    "Spurs",
    "Sq",
    "Sqr",
    "Sqre",
    "Sqrs",
    "Sqs",
    "Squ",
    "Square",
    "Squares",
    "St",
    "Sta",
    "Station",
    "Statn",
    "Stn",
    "Str",
    "Stra",
    "Strav",
    "Straven",
    "Stravenue",
    "Stravn",
    "Stream",
    "Street",
    "Streets",
    "Streme",
    "Strm",
    "Strt",
    "Strvn",
    "Strvnue",
    "Sts",
    "Sumit",
    "Sumitt",
    "Summit",
    "Ter",
    "Terr",
    "Terrace",
    "Throughway",
    "Tpke",
    "Trace",
    "Traces",
    "Track",
    "Tracks",
    "Trafficway",
    "Trail",
    "Trailer",
    "Trails",
    "Trak",
    "Trce",
    "Trfy",
    "Trk",
    "Trks",
    "Trl",
    "Trlr",
    "Trlrs",
    "Trls",
    "Trnpk",
    "Trwy",
    "Tunel",
    "Tunl",
    "Tunls",
    "Tunnel",
    "Tunnels",
    "Tunnl",
    "Turnpike",
    "Turnpk",
    "Un",
    "Underpass",
    "Union",
    "Unions",
    "Uns",
    "Upas",
    "Valley",
    "Valleys",
    "Vally",
    "Vdct",
    "Via",
    "Viadct",
    "Viaduct",
    "View",
    "Views",
    "Vill",
    "Villag",
    "Village",
    "Villages",
    "Ville",
    "Villg",
    "Villiage",
    "Vis",
    "Vist",
    "Vista",
    "Vl",
    "Vlg",
    "Vlgs",
    "Vlly",
    "Vly",
    "Vlys",
    "Vst",
    "Vsta",
    "Vw",
    "Vws",
    "Walk",
    "Walks",
    "Wall",
    "Way",
    "Ways",
    "Well",
    "Wells",
    "Wl",
    "Wls",
    "Wy",
    "Xing",
    "Xrd",
    "Xrds",
]


def street_type_list_to_regex(list_of_street_types: list[str]) -> str:
    """Converts a list of street types into a regex"""
    street_types = "|".join(set(list_of_street_types)).lower()
    for letter in string.ascii_lowercase:
        street_types = street_types.replace(letter, f"[{letter.upper()}{letter}]")

    # Use \b to check that there are word boundaries before and after the street type
    # Optionally match zero to two of " ", ",", or "." after the street name
    street_types = street_types.replace("|", r"\b{div}|\b")
    street_types = r"\b" + street_types + r"\b{div}"
    return street_types.format(
        div=r"[\.\s,]{0,2}",
    )


# Regexp for matching street type
street_type = r"""
            (?:
                (?P<street_type>
                    {street_types}
                )
                (?P<route_id>
                    [\(\s\,]{route_symbols}
                    [Rr][Oo][Uu][Tt][Ee]\s[A-Za-z0-9]+[\)\s\,]{route_symbols}
                )?
            )
""".format(
    route_symbols="{0,3}",
    street_types=street_type_list_to_regex(street_type_list),
)

floor = r"""
            (?P<floor>
                (?:
                \d+[A-Za-z]{0,2}\.?\s[Ff][Ll][Oo][Oo][Rr]\s
                )
                |
                (?:
                    [Ff][Ll][Oo][Oo][Rr]\s\d+[A-Za-z]{0,2}\s
                )
            )
        """

building = rf"""
            (?P<building_id>
                (?:
                    (?:[Bb][Uu][Ii][Ll][Dd][Ii][Nn][Gg])
                    |
                    (?:[Bb][Ll][Dd][Gg])
                )
                \s
                (?:
                    (?:
                        [Aa][Nn][Dd]\s
                        |
                        {thousand}
                        |
                        {hundred}
                        |
                        {zero_to_nine}
                        |
                        {ten_to_ninety}
                    ){{1,5}}
                    |
                    \d{{0,4}}[A-Za-z]?
                )
                \s?
            )
            """

occupancy = r"""
            (?P<occupancy>
                (?:
                    (?:
                        (?:
                            # Suite
                            [Ss][Uu][Ii][Tt][Ee]\s|[Ss][Tt][Ee]\.?\s
                            |
                            # Apartment
                            [Aa][Pp][Tt]\.?\s|[Aa][Pp][Aa][Rr][Tt][Mm][Ee][Nn][Tt]\s
                            |
                            # Room
                            [Rr][Oo][Oo][Mm]\s|[Rr][Mm]\.?\s
                        )
                        (?:
                            [A-Za-z\#\&\-\d]{1,7}
                        )?
                    )
                    |
                    (?:
                        \#[0-9]{,3}[A-Za-z]{1}
                    )
                )\s?
            )
            """

po_box = r"""
            (?:
                [Pp]\.?\s?[Oo]\.?\s[Bb][Oo][Xx]\s\d+
            )
        """

full_street = rf"""
    (?:
        (?P<full_street>
            {street_number}
            {street_name}?\,?\s?
            (?:[\s\,]{street_type})\,?\s?
            {post_direction}?\,?\s?
            {floor}?\,?\s?
            {building}?\,?\s?
            {occupancy}?\,?\s?
            {po_box}?
        )
    )"""

# region1 is actually a "state"
region1 = r"""
        (?P<region1>
            (?:
                # states abbreviations
                AL|AK|AZ|AR|CA|CO|CT|DE|DC|FL|GA|HI|ID|IL|IN|IA|KS|KY|LA|ME|MD|MA|MI|
                MN|MS|MO|MT|NE|NV|NH|NJ|NM|NY|NC|ND|OH|OK|OR|PA|RI|SC|SD|TN|TX|UT|VT|
                VA|WA|WV|WI|WY|
                # unincorporated & commonwealth territories
                AS|GU|MP|PR|VI
            )
            |
            (?:
                # states full
                [Aa][Ll][Aa][Bb][Aa][Mm][Aa]|
                [Aa][Ll][Aa][Ss][Kk][Aa]|
                [Aa][Rr][Ii][Zz][Oo][Nn][Aa]|
                [Aa][Rr][Kk][Aa][Nn][Ss][Aa][Ss]|
                [Cc][Aa][Ll][Ii][Ff][Oo][Rr][Nn][Ii][Aa]|
                [Cc][Oo][Ll][Oo][Rr][Aa][Dd][Oo]|
                [Cc][Oo][Nn][Nn][Ee][Cc][Tt][Ii][Cc][Uu][Tt]|
                [Dd][Ee][Ll][Aa][Ww][Aa][Rr][Ee]|
                [Dd][Ii][Ss][Tt][Rr][Ii][Cc][Tt]\s[Oo][Ff]\s
                [Cc][Oo][Ll][Uu][Mm][Bb][Ii][Aa]|
                [Ff][Ll][Oo][Rr][Ii][Dd][Aa]|
                [Gg][Ee][Oo][Rr][Gg][Ii][Aa]|
                [Hh][Aa][Ww][Aa][Ii][Ii]|
                [Ii][Dd][Aa][Hh][Oo]|
                [Ii][Ll][Ll][Ii][Nn][Oo][Ii][Ss]|
                [Ii][Nn][Dd][Ii][Aa][Nn][Aa]|
                [Ii][Oo][Ww][Aa]|
                [Kk][Aa][Nn][Ss][Aa][Ss]|
                [Kk][Ee][Nn][Tt][Uu][Cc][Kk][Yy]|
                [Ll][Oo][Uu][Ii][Ss][Ii][Aa][Nn][Aa]|
                [Mm][Aa][Ii][Nn][Ee]|
                [Mm][Aa][Rr][Yy][Ll][Aa][Nn][Dd]|
                [Mm][Aa][Ss][Ss][Aa][Cc][Hh][Uu][Ss][Ee][Tt][Tt][Ss]|
                [Mm][Ii][Cc][Hh][Ii][Gg][Aa][Nn]|
                [Mm][Ii][Nn][Nn][Ee][Ss][Oo][Tt][Aa]|
                [Mm][Ii][Ss][Ss][Ii][Ss][Ss][Ii][Pp][Pp][Ii]|
                [Mm][Ii][Ss][Ss][Oo][Uu][Rr][Ii]|
                [Mm][Oo][Nn][Tt][Aa][Nn][Aa]|
                [Nn][Ee][Bb][Rr][Aa][Ss][Kk][Aa]|
                [Nn][Ee][Vv][Aa][Dd][Aa]|
                [Nn][Ee][Ww]\s[Hh][Aa][Mm][Pp][Ss][Hh][Ii][Rr][Ee]|
                [Nn][Ee][Ww]\s[Jj][Ee][Rr][Ss][Ee][Yy]|
                [Nn][Ee][Ww]\s[Mm][Ee][Xx][Ii][Cc][Oo]|
                [Nn][Ee][Ww]\s[Yy][Oo][Rr][Kk]|
                [Nn][Oo][Rr][Tt][Hh]\s[Cc][Aa][Rr][Oo][Ll][Ii][Nn][Aa]|
                [Nn][Oo][Rr][Tt][Hh]\s[Dd][Aa][Kk][Oo][Tt][Aa]|
                [Oo][Hh][Ii][Oo]|
                [Oo][Kk][Ll][Aa][Hh][Oo][Mm][Aa]|
                [Oo][Rr][Ee][Gg][Oo][Nn]|
                [Pp][Ee][Nn][Nn][Ss][Yy][Ll][Vv][Aa][Nn][Ii][Aa]|
                [Rr][Hh][Oo][Dd][Ee]\s[Ii][Ss][Ll][Aa][Nn][Dd]|
                [Ss][Oo][Uu][Tt][Hh]\s[Cc][Aa][Rr][Oo][Ll][Ii][Nn][Aa]|
                [Ss][Oo][Uu][Tt][Hh]\s[Dd][Aa][Kk][Oo][Tt][Aa]|
                [Tt][Ee][Nn][Nn][Ee][Ss][Ss][Ee][Ee]|
                [Tt][Ee][Xx][Aa][Ss]|
                [Uu][Tt][Aa][Hh]|
                [Vv][Ee][Rr][Mm][Oo][Nn][Tt]|
                [Vv][Ii][Rr][Gg][Ii][Nn][Ii][Aa]|
                [Ww][Aa][Ss][Hh][Ii][Nn][Gg][Tt][Oo][Nn]|
                [Ww][Ee][Ss][Tt]\s[Vv][Ii][Rr][Gg][Ii][Nn][Ii][Aa]|
                [Ww][Ii][Ss][Cc][Oo][Nn][Ss][Ii][Nn]|
                [Ww][Yy][Oo][Mm][Ii][Nn][Gg]|
                # unincorporated & commonwealth territories
                [Aa][Mm][Ee][Rr][Ii][Cc][Aa][Nn]\s[Ss][Aa][Mm][Oo][Aa]
                |[Gg][Uu][Aa][Mm]|
                [Nn][Oo][Rr][Tt][Hh][Ee][Rr][Nn]\s[Mm][Aa][Rr][Ii][Aa][Nn][Aa]\s
                [Ii][Ss][Ll][Aa][Nn][Dd][Ss]|
                [Pp][Uu][Ee][Rr][Tt][Oo]\s[Rr][Ii][Cc][Oo]|
                [Vv][Ii][Rr][Gg][Ii][Nn]\s[Ii][Ss][Ll][Aa][Nn][Dd][Ss]
            )
        )
        """

# TODO: doesn't catch cities containing French characters
city = r"""
        (?P<city>
            [A-Za-z]{1}[a-zA-Z\s\-\'\.]{2,20}
        )
        """

postal_code = r"""
            (?P<postal_code>
                (?:\d{5}(?:\-\d{4})?)
            )
            """

country = r"""
            (?:
                [Uu]\.?[Ss]\.?[Aa]\.?|
                [Uu][Nn][Ii][Tt][Ee][Dd]\s[Ss][Tt][Aa][Tt][Ee][Ss](?:\s[Oo][Ff]\s[Aa][Mm][Ee][Rr][Ii][Cc][Aa])?
            )
            """

full_address = r"""
                (?P<full_address>
                    {full_street} {div}
                    {city} {div}
                    {region1} {div}
                    (?:
                        (?:{postal_code}?(\s?,?{country})?)
                    )
                )
                """.format(
    full_street=full_street,
    div=r"[\, ]{,2}",
    city=city,
    region1=region1,
    country=country,
    postal_code=postal_code,
)



"""
pyap.source_GB.data
~~~~~~~~~~~~~~~~~~~~

This module provides regular expression definitions required for
detecting British/GB/UK addresses.

The module is expected to always contain 'full_address' variable containing
all address parsing definitions.

:copyright: (c) 2015 by Vladimir Goncharov.
:license: MIT, see LICENSE for more details.
"""

"""Numerals from one to nine
Note: here and below we use syntax like '[Oo][Nn][Ee]'
instead of '(one)(?i)' to match 'One' or 'oNe' because
Python Regexps don't seem to support turning On/Off
case modes for subcapturing groups.
"""
zero_to_nine = r"""
(?:
    [Zz][Ee][Rr][Oo]\s|[Oo][Nn][Ee]\s|[Tt][Ww][Oo]\s|
    [Tt][Hh][Rr][Ee][Ee]\s|[Ff][Oo][Uu][Rr]\s|
    [Ff][Ii][Vv][Ee]\s|[Ss][Ii][Xx]\s|
    [Ss][Ee][Vv][Ee][Nn]\s|[Ee][Ii][Gg][Hh][Tt]\s|
    [Nn][Ii][Nn][Ee]\s|[Tt][Ee][Nn]\s|
    [Ee][Ll][Ee][Vv][Ee][Nn]\s|
    [Tt][Ww][Ee][Ll][Vv][Ee]\s|
    [Tt][Hh][Ii][Rr][Tt][Ee][Ee][Nn]\s|
    [Ff][Oo][Uu][Rr][Tt][Ee][Ee][Nn]\s|
    [Ff][Ii][Ff][Tt][Ee][Ee][Nn]\s|
    [Ss][Ii][Xx][Tt][Ee][Ee][Nn]\s|
    [Ss][Ee][Vv][Ee][Nn][Tt][Ee][Ee][Nn]\s|
    [Ee][Ii][Gg][Hh][Tt][Ee][Ee][Nn]\s|
    [Nn][Ii][Nn][Ee][Tt][Ee][Ee][Nn]\s
)
"""

# Numerals - 10, 20, 30 ... 90
ten_to_ninety = r"""
(?:
    [Tt][Ee][Nn]\s|[Tt][Ww][Ee][Nn][Tt][Yy]\s|
    [Tt][Hh][Ii][Rr][Tt][Yy]\s|
    [Ff][Oo][Rr][Tt][Yy]\s|
    [Ff][Oo][Uu][Rr][Tt][Yy]\s|
    [Ff][Ii][Ff][Tt][Yy]\s|[Ss][Ii][Xx][Tt][Yy]\s|
    [Ss][Ee][Vv][Ee][Nn][Tt][Yy]\s|
    [Ee][Ii][Gg][Hh][Tt][Yy]\s|
    [Nn][Ii][Nn][Ee][Tt][Yy]\s
)
"""

# One hundred
hundred = r"""
(?:
    [Hh][Uu][Nn][Dd][Rr][Ee][Dd]\s
)
"""

# One thousand
thousand = r"""
(?:
    [Tt][Hh][Oo][Uu][Ss][Aa][Nn][Dd]\s
)
"""

part_divider = r"(?: [\,\s\.\-]{0,3}\,[\,\s\.\-]{0,3} )"
space_pattern = r"(?: [\s\t]{1,3} )"  # TODO: use \b for word boundary

"""
Regexp for matching street number.
Street number can be written 2 ways:
1) Using letters - "One thousand twenty two"
2) Using numbers
   a) - "1022"
   b) - "85-1190"
   c) - "85 1190"
"""
street_number = r"""
(?P<street_number>
    (?:
        (?:
            [Nn][Uu][Mm][Bb][Ee][Rr]|
            [Nn][RrOo]\.?|
            [Nn][Uu][Mm]\.?|
            #
        )
        {space}?
    )?
    (?:
        (?:
            [Aa][Nn][Dd]\s
            |
            {thousand} 
            |
            {hundred} 
            |
            {zero_to_nine} 
            |
            {ten_to_ninety} 
        ){from_to}
        |
        (?:
            \d{from_to}
            (?: {space}? [A-Za-z] (?![A-Za-z\d]]) )?
            (?!\d)
            (?:{space}?\-{space}?\d{from_to} (?: {space}? [A-Za-z] (?![A-Za-z\d]) )? )?
        )
    )
    {space}?
)  # end street_number
""".format(
    thousand=thousand,
    hundred=hundred,
    zero_to_nine=zero_to_nine,
    ten_to_ninety=ten_to_ninety,
    space=space_pattern,
    from_to="{1,5}",
)

"""
Regexp for matching street name.
In "Hoover Boulevard", "Hoover" is a street name
Seems like the longest US street is 'Northeast Kentucky Industrial Parkway',
which is 31 charactors
https://atkinsbookshelf.wordpress.com/tag/longest-street-name-in-us/
"""
street_name = r"""
(?P<street_name>
    (?(street_number)           # If street_number has been found, then digits
                                # can be in the street otherwise no digits are
        [a-zA-Z0-9\s\.]{3,31}   # allowed. This aims to prevent street_name
        |                       # matching everything before the address as
        [a-zA-Z\s\.]{3,31}      # well as the number.
    )
)
"""

post_direction = r"""
(?P<post_direction>
    (?:
        [Nn][Oo][Rr][Tt][Hh]\s|
        [Ss][Oo][Uu][Tt][Hh]\s|
        [Ee][Aa][Ss][Tt]\s |
        [Ww][Ee][Ss][Tt]\s 
    )
    |
    (?:
        NW\s|NE\s|SW\s|SE\s
    )
    |
    (?:
        N\.?\s|S\.?\s|E\.?\s|W\.?\s
    )
)  # end post_direction
"""

# Regexp for matching street type
street_type = r"""
(?:
    (?P<street_type>
        # Street
        [Ss][Tt][Rr][Ee][Ee][Tt]|S[Tt]\.?(?![A-Za-z])|
        # Boulevard
        [Bb][Oo][Uu][Ll][Ee][Vv][Aa][Rr][Dd]|[Bb][Ll][Vv][Dd]\.?|
        # Highway
        [Hh][Ii][Gg][Hh][Ww][Aa][Yy]|H[Ww][Yy]\.?|
        # Broadway
        [Bb][Rr][Oo][Aa][Dd][Ww][Aa][Yy]|
        # Freeway
        [Ff][Rr][Ee][Ee][Ww][Aa][Yy]|
        # Causeway
        [Cc][Aa][Uu][Ss][Ee][Ww][Aa][Yy]|C[Ss][Ww][Yy]\.?|
        # Expressway
        [Ee][Xx][Pp][Rr][Ee][Ss][Ss][Ww][Aa][Yy]|
        # Way
        [Ww][Aa][Yy]|
        # Walk
        [Ww][Aa][Ll][Kk]|
        # Lane
        [Ll][Aa][Nn][Ee]|L[Nn]\.?|
        # Road
        [Rr][Oo][Aa][Dd]|R[Dd]\.?|
        # Avenue
        [Aa][Vv][Ee][Nn][Uu][Ee]|A[Vv][Ee]\.?|
        # Circle
        [Cc][Ii][Rr][Cc][Ll][Ee]|C[Ii][Rr]\.?|
        # Cove
        [Cc][Oo][Vv][Ee]|C[Vv]\.?|
        # Drive
        [Dd][Rr][Ii][Vv][Ee]|D[Rr]\.?|
        # Parkway
        [Pp][Aa][Rr][Kk][Ww][Aa][Yy]|P[Kk][Ww][Yy]\.?|
        # Park
        [Pp][Aa][Rr][Kk]|
        # Court
        [Cc][Oo][Uu][Rr][Tt]|C[Tt]\.?|
        # Square
        [Ss][Qq][Uu][Aa][Rr][Ee]|S[Qq]\.?|
        # Loop
        [Ll][Oo][Oo][Pp]|L[Pp]\.?|
        # Place
        [Pp][Ll][Aa][Cc][Ee]|P[Ll]\.?|
        # Parade
        [Pp][Aa][Rr][Aa][Dd][Ee]|P[Ll]\.?|
        # Estate
        [Ee][Ss][Tt][Aa][Tt][Ee]
    )
    (?P<route_id>)
)  # end street_type
"""

floor = r"""
(?P<floor>
    (?:
    \d+[A-Za-z]{0,2}\.?\s[Ff][Ll][Oo][Oo][Rr]\s
    )
    |
    (?:
        [Ff][Ll][Oo][Oo][Rr]\s\d+[A-Za-z]{0,2}\s
    )
)  # end floor
"""

building = rf"""
(?P<building_id>
    (?:
        (?:[Bb][Uu][Ii][Ll][Dd][Ii][Nn][Gg])
        |
        (?:[Bb][Ll][Dd][Gg])
    )
    \s
    (?:
        (?:
            [Aa][Nn][Dd]\s
            |
            {thousand}
            |
            {hundred}
            |
            {zero_to_nine}
            |
            {ten_to_ninety}
        ){{1,5}}
        |
        \d{{0,4}}[A-Za-z]?
    )
    \s ?
)  # end building_id
"""

occupancy = rf"""
(?P<occupancy>
    (?:
        (?:
            # Suite
            [Ss][Uu][Ii][Tt][Ee]|[Ss][Tt][Ee]\.?
            |
            # Studio
            [Ss][Tt][Uu][Dd][Ii][Oo]|[Ss][Tt][UuDd]\.?
            |
            # Apartment
            [Aa][Pp][Tt]\.?|[Aa][Pp][Aa][Rr][Tt][Mm][Ee][Nn][Tt]
            |
            # Room
            [Rr][Oo][Oo][Mm]|[Rr][Mm]\.?
            |
            # Flat
            [Ff][Ll][Aa][Tt]
            |
            \#
        )
        {space_pattern}?
        (?:
            [A-Za-z\#\&\-\d]{{1,7}}
        )?
    )
    {space_pattern}?
)  # end occupancy
"""

po_box = rf"""
(?:
    [Pp]\.? {space_pattern}? [Oo]\.? {space_pattern}? ([Bb][Oo][Xx]{space_pattern}?)?\d+
)
"""

# TODO: maybe remove the '?' on the part_dividers is mismatch address parts
full_street = rf"""
(?:
    (?P<full_street>

        (?:
            {po_box} {part_divider}?
        )?
        (?:
            {floor} {part_divider}?
        )?
        (?:
            {occupancy} {part_divider}?
        )?
        (?:
            {building} {part_divider}?
        )?

        (?:
            (?: {street_number} {space_pattern} )
            |
            (?! \d{{}} )

        )?
        (?:{street_name} )
        (?:{space_pattern} {street_type} {space_pattern}?)?
    )
)  # end full_street
"""

# region1 is actually a "state"
region1 = r"""
(?P<region1>
    [A-Za-z]{1}[a-zA-Z0-9\s\.\-']{1,35}
)  # end region1
"""

city = r"""
(?P<city>
    [A-Za-z]{1}[a-zA-Z0-9\s\.\-']{1,35}
)  # end city
"""

postal_code = r"""
(?P<postal_code>
    (?:  # Mainland British postcodes
        (?:
            (?:\b[Ww][Cc][0-9][abehmnprvwxyABEHMNPRVWXY])|
            (?:\b[Ee][Cc][1-4][abehmnprvwxyABEHMNPRVWXY])|
            (?:\b[Nn][Ww]1[Ww])|
            (?:\b[Ss][Ee]1[Pp])|
            (?:\b[Ss][Ww]1[abehmnprvwxyABEHMNPRVWXY])|
            (?:\b[EeNnWw]1[a-hjkpstuwA-HJKPSTUW])|
            (?:\b[BbEeGgLlMmNnSsWw][0-9][0-9]?)|
            (?:\b[a-pr-uwyzA-PR-UWYZ][a-hk-yxA-HK-XY][0-9][0-9]?)
        )
        \s{0,}[0-9][abd-hjlnp-uw-zABD-HJLNP-UW-Z]{2}
    )
)  # end postal_code
"""

country = r"""
(?P<country>
    (?:[Tt][Hh][Ee]\s*)?[Uu][Nn][Ii][Tt][Ee][Dd]\s*[Kk][Ii][Nn][Gg][Dd][Oo][Mm]\s*[Oo][Ff]\s*(?:[Gg][Rr][Ee][Aa][Tt]\s*)?[Bb][Rr][Ii][Tt][Aa][Ii][Nn](?:\s*[Aa][Nn][Dd]\s*[Nn][Oo][Rr][Tt][Hh][Ee][Rr][Nn]\s*[Ii][Rr][Ee][Ll][Aa][Nn][Dd])?|
    (?:[Gg][Rr][Ee][Aa][Tt]\s*)?[Bb][Rr][Ii][Tt][Aa][Ii][Nn](?:\s*[Aa][Nn][Dd]\s*[Nn][Oo][Rr][Tt][Hh][Ee][Rr][Nn]\s*[Ii][Rr][Ee][Ll][Aa][Nn][Dd])?|
    (?:[Tt][Hh][Ee]\s*)?[Uu][Nn][Ii][Tt][Ee][Dd]\s*[Kk][Ii][Nn][Gg][Dd][Oo][Mm]|
    (?:[Nn][Oo][Rr][Tt][Hh][Ee][Rr][Nn]\s*)?[Ii][Rr][Ee][Ll][Aa][Nn][Dd]|
    [Ee][Nn][Gg][Ll][Aa][Nn][Dd]|
    [Ss][Cc][Oo][Tt][Ll][Aa][Nn][Dd]|
    [Ww][Aa][Ll][Ee][Ss]|
    [Cc][Yy][Mm][Rr][Uu]|
    [Gg][Bb]|
    [Uu][Kk]|
    [Nn]\.?\s*[Ii]\.?
)  # end country
"""

full_address = rf"""
(?P<full_address>
    {full_street}
    (?: {part_divider} {city} )?
    (?: {part_divider} {region1} )?
    {part_divider}? {postal_code}
    (?: {part_divider} {country} )?
)  # end full_address
"""



"""
pyap.api
~~~~~~~~~~~~~~~~

This module contains address parser API functions.

:copyright: (c) 2015 by Vladimir Goncharov.
:license: MIT, see LICENSE for more details.
"""

from typing import Any

from pyap_beauhurst.address import Address

from . import parser


def parse(some_text: str, **kwargs: Any) -> list[Address | None]:
    """Creates request to AddressParser
    and returns list of Address objects
    """
    ap = parser.AddressParser(**kwargs)
    return ap.parse(some_text)
